==> ponyc-0.18.0-1-x86_64.pkg.tar.xz: usr/share/doc/pony/stdlib-docs/docs/collections-persistent-Vec.md <==
# Vec\[A: [Any](builtin-Any) #share\]

A persistent vector based on the Hash Array Mapped Trie from 'Ideal Hash
Trees' by Phil Bagwell.


```pony
class val Vec[A: Any #share]
```

## Constructors

### create

```pony
new val create()
: Vec[A #share] val^
```

#### Returns

* [Vec](collections-persistent-Vec)\[A #share\] val^

---

### _create

```pony
new val _create(
  root': (_VecNode[A #share] val | None val),
  tail': Array[A #share] val,
  size': USize val,
  depth': U8 val,
  tail_offset': USize val)
: Vec[A #share] val^
```
#### Parameters

*   root': ([_VecNode](collections-persistent-_VecNode)\[A #share\] val | [None](builtin-None) val)
*   tail': [Array](builtin-Array)\[A #share\] val
*   size': [USize](builtin-USize) val
*   depth': [U8](builtin-U8) val
*   tail_offset': [USize](builtin-USize) val

#### Returns

* [Vec](collections-persistent-Vec)\[A #share\] val^

---

## Public Functions

### size

Return the amount of values in the vector.


```pony
fun box size()
: USize val
```

#### Returns

* [USize](builtin-USize) val

---

### apply

Get the i-th element, raising an error if the index is out of bounds.


```pony
fun box apply(
  i: USize val)
: A #share ?
```
#### Parameters

*   i: [USize](builtin-USize) val

#### Returns

* A #share ?

---

### update

Return a vector with the i-th element changed, raising an error if the
index is out of bounds.


```pony
fun val update(
  i: USize val,
  value: A #share)
: Vec[A #share] val ?
```
#### Parameters

*   i: [USize](builtin-USize) val
*   value: A #share

#### Returns

* [Vec](collections-persistent-Vec)\[A #share\] val ?

---

### insert

Return a vector with an element inserted. Elements after this are moved
up by one index, extending the vector. An out of bounds index raises an
error.


```pony
fun val insert(
  i: USize val,
  value: A #share)
: Vec[A #share] val ?
```
#### Parameters

*   i: [USize](builtin-USize) val
*   value: A #share

#### Returns

* [Vec](collections-persistent-Vec)\[A #share\] val ?

---

### delete

Return a vector with an element deleted. Elements after this are moved
down by one index, compacting the vector. An out of bounds index raises an
error.


```pony
fun val delete(
  i: USize val)
: Vec[A #share] val ?
```
#### Parameters

*   i: [USize](builtin-USize) val

#### Returns

* [Vec](collections-persistent-Vec)\[A #share\] val ?

---

### remove

Return a vector with n elements removed, beginning at index i.


```pony
fun val remove(
  i: USize val,
  n: USize val)
: Vec[A #share] val ?
```
#### Parameters

*   i: [USize](builtin-USize) val
*   n: [USize](builtin-USize) val

#### Returns

* [Vec](collections-persistent-Vec)\[A #share\] val ?

---

### push

Return a vector with the value added to the end.


```pony
fun val push(
  value: A #share)
: Vec[A #share] val
```
#### Parameters

*   value: A #share

#### Returns

* [Vec](collections-persistent-Vec)\[A #share\] val

---

### pop

Return a vector with the value at the end removed.


```pony
fun val pop()
: Vec[A #share] val ?
```

#### Returns

* [Vec](collections-persistent-Vec)\[A #share\] val ?

---

### concat

Return a vector with the values of the given iterator added to the end.


```pony
fun val concat(
  iter: Iterator[A #share] ref)
: Vec[A #share] val
```
#### Parameters

*   iter: [Iterator](builtin-Iterator)\[A #share\] ref

#### Returns

* [Vec](collections-persistent-Vec)\[A #share\] val

---

### find

Find the `nth` appearance of `value` from the beginning of the vector,
starting at `offset` and examining higher indices, and using the
supplied `predicate` for comparisons. Returns the index of the value, or
raise an error if the value isn't present.

By default, the search starts at the first element of the vector,
returns the first instance of `value` found, and uses object identity
for comparison.


```pony
fun val find(
  value: A #share,
  offset: USize val = 0,
  nth: USize val = 0,
  predicate: {(A, A): Bool}[A #share] val = lambda)
: USize val ?
```
#### Parameters

*   value: A #share
*   offset: [USize](builtin-USize) val = 0
*   nth: [USize](builtin-USize) val = 0
*   predicate: {(A, A): Bool}[A #share] val = lambda

#### Returns

* [USize](builtin-USize) val ?

---

### contains

Returns true if the vector contains `value`, false otherwise.


```pony
fun val contains(
  value: A #share,
  predicate: {(A, A): Bool}[A #share] val = lambda)
: Bool val
```
#### Parameters

*   value: A #share
*   predicate: {(A, A): Bool}[A #share] val = lambda

#### Returns

* [Bool](builtin-Bool) val

---

### slice

Return a vector that is a clone of a portion of this vector. The range is
exclusive and saturated.


```pony
fun val slice(
  from: USize val = 0,
  to: USize val = call,
  step: USize val = 1)
: Vec[A #share] val
```
#### Parameters

*   from: [USize](builtin-USize) val = 0
*   to: [USize](builtin-USize) val = call
*   step: [USize](builtin-USize) val = 1

#### Returns

* [Vec](collections-persistent-Vec)\[A #share\] val

---

### reverse

Return a vector with the elements in reverse order.


```pony
fun val reverse()
: Vec[A #share] val
```

#### Returns

* [Vec](collections-persistent-Vec)\[A #share\] val

---

### keys

Return an iterator over the indices in the vector.


```pony
fun val keys()
: VecKeys[A #share] ref^
```

#### Returns

* [VecKeys](collections-persistent-VecKeys)\[A #share\] ref^

---

### values

Return an iterator over the values in the vector.


```pony
fun val values()
: VecValues[A #share] ref^
```

#### Returns

* [VecValues](collections-persistent-VecValues)\[A #share\] ref^

---

### pairs

Return an iterator over the (index, value) pairs in the vector.


```pony
fun val pairs()
: VecPairs[A #share] ref^
```

#### Returns

* [VecPairs](collections-persistent-VecPairs)\[A #share\] ref^

---

## Private Functions

### _pow32

Raise 32 to the power of n.


```pony
fun box _pow32(
  n: USize val)
: USize val
```
#### Parameters

*   n: [USize](builtin-USize) val

#### Returns

* [USize](builtin-USize) val

---

### _leaf_nodes

```pony
fun box _leaf_nodes()
: Array[Array[A #share] val] ref^
```

#### Returns

* [Array](builtin-Array)\[[Array](builtin-Array)\[A #share\] val\] ref^

---

==> ponyc-0.18.1-1-x86_64.pkg.tar.xz: usr/share/doc/pony/stdlib-docs/docs/collections-persistent-Vec.md <==
# Vec\[A: [Any](builtin-Any) #share\]

A persistent vector based on the Hash Array Mapped Trie from 'Ideal Hash
Trees' by Phil Bagwell.


```pony
class val Vec[A: Any #share]
```

## Constructors

### create

```pony
new val create()
: Vec[A] val^
```

#### Returns

* [Vec](collections-persistent-Vec)\[A\] val^

---

### _create

```pony
new val _create(
  root': (_VecNode[A] val | None val),
  tail': Array[A] val,
  size': USize val,
  depth': U8 val,
  tail_offset': USize val)
: Vec[A] val^
```
#### Parameters

*   root': ([_VecNode](collections-persistent-_VecNode)\[A\] val | [None](builtin-None) val)
*   tail': [Array](builtin-Array)\[A\] val
*   size': [USize](builtin-USize) val
*   depth': [U8](builtin-U8) val
*   tail_offset': [USize](builtin-USize) val

#### Returns

* [Vec](collections-persistent-Vec)\[A\] val^

---

## Public Functions

### size

Return the amount of values in the vector.


```pony
fun box size()
: USize val
```

#### Returns

* [USize](builtin-USize) val

---

### apply

Get the i-th element, raising an error if the index is out of bounds.


```pony
fun box apply(
  i: USize val)
: A ?
```
#### Parameters

*   i: [USize](builtin-USize) val

#### Returns

* A ?

---

### update

Return a vector with the i-th element changed, raising an error if the
index is out of bounds.


```pony
fun val update(
  i: USize val,
  value: A)
: Vec[A] val ?
```
#### Parameters

*   i: [USize](builtin-USize) val
*   value: A

#### Returns

* [Vec](collections-persistent-Vec)\[A\] val ?

---

### insert

Return a vector with an element inserted. Elements after this are moved
up by one index, extending the vector. An out of bounds index raises an
error.


```pony
fun val insert(
  i: USize val,
  value: A)
: Vec[A] val ?
```
#### Parameters

*   i: [USize](builtin-USize) val
*   value: A

#### Returns

* [Vec](collections-persistent-Vec)\[A\] val ?

---

### delete

Return a vector with an element deleted. Elements after this are moved
down by one index, compacting the vector. An out of bounds index raises an
error.


```pony
fun val delete(
  i: USize val)
: Vec[A] val ?
```
#### Parameters

*   i: [USize](builtin-USize) val

#### Returns

* [Vec](collections-persistent-Vec)\[A\] val ?

---

### remove

Return a vector with n elements removed, beginning at index i.


```pony
fun val remove(
  i: USize val,
  n: USize val)
: Vec[A] val ?
```
#### Parameters

*   i: [USize](builtin-USize) val
*   n: [USize](builtin-USize) val

#### Returns

* [Vec](collections-persistent-Vec)\[A\] val ?

---

### push

Return a vector with the value added to the end.


```pony
fun val push(
  value: A)
: Vec[A] val
```
#### Parameters

*   value: A

#### Returns

* [Vec](collections-persistent-Vec)\[A\] val

---

### pop

Return a vector with the value at the end removed.


```pony
fun val pop()
: Vec[A] val ?
```

#### Returns

* [Vec](collections-persistent-Vec)\[A\] val ?

---

### concat

Return a vector with the values of the given iterator added to the end.


```pony
fun val concat(
  iter: Iterator[A] ref)
: Vec[A] val
```
#### Parameters

*   iter: [Iterator](builtin-Iterator)\[A\] ref

#### Returns

* [Vec](collections-persistent-Vec)\[A\] val

---

### find

Find the `nth` appearance of `value` from the beginning of the vector,
starting at `offset` and examining higher indices, and using the
supplied `predicate` for comparisons. Returns the index of the value, or
raise an error if the value isn't present.

By default, the search starts at the first element of the vector,
returns the first instance of `value` found, and uses object identity
for comparison.


```pony
fun val find(
  value: A,
  offset: USize val = 0,
  nth: USize val = 0,
  predicate: {(A, A): Bool}[A] val = lambda)
: USize val ?
```
#### Parameters

*   value: A
*   offset: [USize](builtin-USize) val = 0
*   nth: [USize](builtin-USize) val = 0
*   predicate: {(A, A): Bool}[A] val = lambda

#### Returns

* [USize](builtin-USize) val ?

---

### contains

Returns true if the vector contains `value`, false otherwise.


```pony
fun val contains(
  value: A,
  predicate: {(A, A): Bool}[A] val = lambda)
: Bool val
```
#### Parameters

*   value: A
*   predicate: {(A, A): Bool}[A] val = lambda

#### Returns

* [Bool](builtin-Bool) val

---

### slice

Return a vector that is a clone of a portion of this vector. The range is
exclusive and saturated.


```pony
fun val slice(
  from: USize val = 0,
  to: USize val = call,
  step: USize val = 1)
: Vec[A] val
```
#### Parameters

*   from: [USize](builtin-USize) val = 0
*   to: [USize](builtin-USize) val = call
*   step: [USize](builtin-USize) val = 1

#### Returns

* [Vec](collections-persistent-Vec)\[A\] val

---

### reverse

Return a vector with the elements in reverse order.


```pony
fun val reverse()
: Vec[A] val
```

#### Returns

* [Vec](collections-persistent-Vec)\[A\] val

---

### keys

Return an iterator over the indices in the vector.


```pony
fun val keys()
: VecKeys[A] ref^
```

#### Returns

* [VecKeys](collections-persistent-VecKeys)\[A\] ref^

---

### values

Return an iterator over the values in the vector.


```pony
fun val values()
: VecValues[A] ref^
```

#### Returns

* [VecValues](collections-persistent-VecValues)\[A\] ref^

---

### pairs

Return an iterator over the (index, value) pairs in the vector.


```pony
fun val pairs()
: VecPairs[A] ref^
```

#### Returns

* [VecPairs](collections-persistent-VecPairs)\[A\] ref^

---

## Private Functions

### _pow32

Raise 32 to the power of n.


```pony
fun box _pow32(
  n: USize val)
: USize val
```
#### Parameters

*   n: [USize](builtin-USize) val

#### Returns

* [USize](builtin-USize) val

---

### _leaf_nodes

```pony
fun box _leaf_nodes()
: Array[Array[A] val] ref^
```

#### Returns

* [Array](builtin-Array)\[[Array](builtin-Array)\[A\] val\] ref^

---
